@@ -1,33 +1,44 @@
 /*
- *  $Id: combined_server.c 742 2005-09-01 15:44:24Z tuexen $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
+ * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
+ *          andreas.jungmaier@web.de
  */
 
 #ifndef WIN32
@@ -73,7 +84,7 @@ static int unknownCommand                     = 0;
 static int sendOOTBAborts                     = 1;
 static int timeToLive                         = SCTP_INFINITE_LIFETIME;
 
-char *
+const char *
 pathStateName(unsigned int state)
 {
     switch (state) {
@@ -109,56 +120,56 @@ void printUsage(void)
     printf("-s       source address\n");
     printf("-t       time to live in ms\n");
     printf("-v       verbose mode\n");
-    printf("-V       very verbose mode\n");   
+    printf("-V       very verbose mode\n");
 }
 
 void getArgs(int argc, char **argv)
 {
     int i;
     char *opt;
-    
+
     for(i=1; i < argc ;i++) {
         if (argv[i][0] == '-') {
             switch (argv[i][1]) {
                 case 'h':
-				    printUsage();
-				    exit(0);  
+                  printUsage();
+                  exit(0);
                 case 's':
-				    if (i+1 >= argc) {
-                        printUsage();
-				        exit(0);
-				    }
-				    opt = argv[++i];
-				    if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) && (strlen(opt) < SCTP_MAX_IP_LEN  )) {
-				        strcpy((char *)localAddressList[noOfLocalAddresses], opt);
-				        noOfLocalAddresses++;
-				    };
-				    break;
+                  if (i+1 >= argc) {
+                     printUsage();
+                     exit(0);
+                  }
+                  opt = argv[++i];
+                  if ((noOfLocalAddresses < MAXIMUM_NUMBER_OF_LOCAL_ADDRESSES) && (strlen(opt) < SCTP_MAX_IP_LEN  )) {
+                     strcpy((char *)localAddressList[noOfLocalAddresses], opt);
+                     noOfLocalAddresses++;
+                  };
+                break;
                 case 'i':
-				    sendOOTBAborts = 0;
-				    break;
+                  sendOOTBAborts = 0;
+                  break;
                 case 't':
-				    if (i+1 >= argc) {
-                        printUsage();
-						exit(0);
-				    }
-				    opt=argv[++i];
-				    timeToLive = atoi(opt);
-				    break;
+                  if (i+1 >= argc) {
+                     printUsage();
+                     exit(0);
+                  }
+                  opt=argv[++i];
+                  timeToLive = atoi(opt);
+                break;
                 case 'v':
-				    verbose = 1;
-				    break;
+                  verbose = 1;
+                break;
                 case 'V':
-				    verbose = 1;
-				    vverbose = 1;
-				    break;
+                  verbose = 1;
+                  vverbose = 1;
+                break;
                 default:
-				    unknownCommand = 1;
-				    break;
-			}			
+                  unknownCommand = 1;
+                break;
+         }
         }
-		else
-			unknownCommand = 1;
+      else
+         unknownCommand = 1;
     }
 }
 
@@ -166,10 +177,10 @@ void checkArgs(void)
 {
     int abortProgram;
     int printUsageInfo;
-    
+
     abortProgram = 0;
     printUsageInfo = 0;
-    
+
     if (noOfLocalAddresses == 0) {
 #ifdef HAVE_IPV6
         strcpy((char *)localAddressList[noOfLocalAddresses], "::0");
@@ -182,7 +193,7 @@ void checkArgs(void)
          printf("Error:   Unkown options in command.\n");
          printUsageInfo = 1;
     }
-    
+
     if (printUsageInfo == 1)
         printUsage();
     if (abortProgram == 1)
@@ -197,7 +208,7 @@ void echoDataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned
     unsigned short ssn;
     unsigned int tsn;
 
-    if (vverbose) {  
+    if (vverbose) {
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
@@ -222,7 +233,7 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
     unsigned short ssn;
     unsigned int tsn;
 
-    if (vverbose) {  
+    if (vverbose) {
       fprintf(stdout, "%-8x: Data arrived (%u bytes on stream %u, %s)\n",
                       assocID, len, streamID, (unordered==SCTP_ORDERED_DELIVERY)?"ordered":"unordered");
       fflush(stdout);
@@ -235,7 +246,7 @@ void dataArriveNotif(unsigned int assocID, unsigned short streamID, unsigned int
 void sendFailureNotif(unsigned int assocID,
                       unsigned char *unsent_data, unsigned int dataLength, unsigned int *context, void* dummy)
 {
-  if (verbose) {  
+  if (verbose) {
     fprintf(stdout, "%-8x: Send failure\n", assocID);
     fflush(stdout);
   }
@@ -246,20 +257,20 @@ void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsign
     SCTP_AssociationStatus assocStatus;
     SCTP_PathStatus pathStatus;
     unsigned short pathID;
-    
+
     if (verbose) {
         SCTP_getPathStatus(assocID, affectedPathID, &pathStatus);
-        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n", 
+        fprintf(stdout, "%-8x: Network status change: path %u (towards %s) is now %s\n",
                         assocID, affectedPathID,
                         pathStatus.destinationAddress,
                         pathStateName(newState));
         fflush(stdout);
     }
-    
+
     /* if the primary path has become inactive */
     if ((newState == SCTP_PATH_UNREACHABLE) &&
         (affectedPathID == SCTP_getPrimary(assocID))) {
-        
+
         /* select a new one */
         SCTP_getAssocStatus(assocID, &assocStatus);
         for (pathID=0; pathID < assocStatus.numberOfAddresses; pathID++){
@@ -267,7 +278,7 @@ void networkStatusChangeNotif(unsigned int assocID, short affectedPathID, unsign
             if (pathStatus.state == SCTP_PATH_OK)
                 break;
         }
-        
+
         /* and use it */
         if (pathID < assocStatus.numberOfAddresses) {
             SCTP_setPrimary(assocID, pathID);
@@ -280,24 +291,24 @@ void* echoCommunicationUpNotif(unsigned int assocID, int status,
                                unsigned int noOfPaths,
                                unsigned short noOfInStreams, unsigned short noOfOutStreams,
                                int associationSupportsPRSCTP, void* dummy)
-{	
+{
     struct ulp_data *ulpDataPtr;
     SCTP_PathStatus pathStatus;
     unsigned int i;
 
-    if (verbose) {  
+    if (verbose) {
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
         fflush(stdout);
     }
-    
+
     if (vverbose) {
         for (i=0; i < noOfPaths; i++) {
             SCTP_getPathStatus(assocID, (short)i, &pathStatus);
             fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
         }
     }
-  
-    ulpDataPtr                   = malloc(sizeof(struct ulp_data));
+
+    ulpDataPtr                   = (struct ulp_data*)malloc(sizeof(struct ulp_data));
     ulpDataPtr->maximumStreamID  = noOfOutStreams - 1;
     ulpDataPtr->ShutdownReceived = 0;
     return((void *) ulpDataPtr);
@@ -307,40 +318,40 @@ void* daytimeCommunicationUpNotif(unsigned int assocID, int status,
                                   unsigned int noOfPaths,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
                                   int associationSupportsPRSCTP, void* dummy)
-{	
+{
     char *timeAsString;
     time_t now;
     SCTP_PathStatus pathStatus;
     unsigned int i;
- 
-    if (verbose) {  
+
+    if (verbose) {
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
         fflush(stdout);
     }
-    
+
     if (vverbose) {
         for (i=0; i < noOfPaths; i++) {
             SCTP_getPathStatus(assocID, (short)i, &pathStatus);
             fprintf(stdout, "%-8x: Path Status of path %u (towards %s): %s.\n", assocID, i, pathStatus.destinationAddress, pathStateName(pathStatus.state));
         }
     }
-   
+
     /* get the current time and convert to string */
     time(&now);
     timeAsString = ctime(&now);
 
-    if (vverbose) {  
+    if (vverbose) {
         fprintf(stdout, "%-8x: Current Time: %s", assocID, timeAsString);
         fflush(stdout);
     }
-   
+
     SCTP_send(assocID,
               0,
               (unsigned char *)timeAsString, strlen(timeAsString),
               SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, 
-	          timeToLive, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
-    
+              SCTP_USE_PRIMARY, SCTP_NO_CONTEXT,
+             timeToLive, SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED);
+
     SCTP_shutdown(assocID);
 
     return NULL;
@@ -350,18 +361,18 @@ void* chargenCommunicationUpNotif(unsigned int assocID, int status,
                                   unsigned int noOfPaths,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
                                   int associationSupportsPRSCTP, void* dummy)
-{	
+{
     int length;
     unsigned char buffer[BUFFER_LENGTH];
     struct ulp_data *ulpDataPtr;
     SCTP_PathStatus pathStatus;
     unsigned int i;
 
-    if (verbose) {  
+    if (verbose) {
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
         fflush(stdout);
     }
-    
+
     if (vverbose) {
         for (i=0; i < noOfPaths; i++) {
             SCTP_getPathStatus(assocID, (short)i, &pathStatus);
@@ -372,9 +383,9 @@ void* chargenCommunicationUpNotif(unsigned int assocID, int status,
     length = 1 + (rand() % 512);
     memset((void *)buffer, 'A', length);
     buffer[length-1] = '\n';
-    
+
     while(SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                    SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
+                    SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
                     SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS) {
       if (vverbose) {
           fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
@@ -384,7 +395,7 @@ void* chargenCommunicationUpNotif(unsigned int assocID, int status,
       memset(buffer, 'A', length);
       buffer[length-1] = '\n';
     }
-    ulpDataPtr                   = malloc(sizeof(struct ulp_data));
+    ulpDataPtr                   = (struct ulp_data*)malloc(sizeof(struct ulp_data));
     ulpDataPtr->maximumStreamID  = noOfOutStreams - 1;
     ulpDataPtr->ShutdownReceived = 0;
     return((void *) ulpDataPtr);
@@ -394,15 +405,15 @@ void* discardCommunicationUpNotif(unsigned int assocID, int status,
                                   unsigned int noOfPaths,
                                   unsigned short noOfInStreams, unsigned short noOfOutStreams,
                                   int associationSupportsPRSCTP, void* dummy)
-{	
+{
     SCTP_PathStatus pathStatus;
     unsigned int i;
 
-    if (verbose) {  
+    if (verbose) {
         fprintf(stdout, "%-8x: Communication up (%u paths)\n", assocID, noOfPaths);
         fflush(stdout);
     }
-        
+
     if (vverbose) {
         for (i=0; i < noOfPaths; i++) {
             SCTP_getPathStatus(assocID, (short)i, &pathStatus);
@@ -414,7 +425,7 @@ void* discardCommunicationUpNotif(unsigned int assocID, int status,
 }
 
 void communicationLostNotif(unsigned int assocID, unsigned short status, void* ulpDataPtr)
-{	
+{
     unsigned char buffer[SCTP_MAXIMUM_DATA_LENGTH];
     unsigned int bufferLength;
     unsigned short streamID, streamSN;
@@ -425,7 +436,7 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         fprintf(stdout, "%-8x: Communication lost (status %u)\n", assocID, status);
         fflush(stdout);
     }
-  
+
     /* retrieve data */
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnsent(assocID, buffer, &bufferLength, &tsn,
@@ -438,10 +449,10 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-    
+
     bufferLength = sizeof(buffer);
     while (SCTP_receiveUnacked(assocID, buffer, &bufferLength, &tsn,
-			       &streamID, &streamSN, &protoID) >= 0){
+                               &streamID, &streamSN, &protoID) >= 0){
         /* do something with the retrieved data */
         if (vverbose) {
             fprintf(stdout, "%-8x: Retrieved unacked chunk with %u bytes (SSN %u, SID %u)\n", assocID,bufferLength,streamSN,streamID );
@@ -450,17 +461,17 @@ void communicationLostNotif(unsigned int assocID, unsigned short status, void* u
         /* after that, reset bufferLength */
         bufferLength = sizeof(buffer);
     }
-                          
+
     /* delete the association */
     if (ulpDataPtr)
         free((struct ulp_data *) ulpDataPtr);
-        
+
     SCTP_deleteAssociation(assocID);
 }
 
 void communicationErrorNotif(unsigned int assocID, unsigned short status, void* dummy)
 {
-  if (verbose) {  
+  if (verbose) {
     fprintf(stdout, "%-8x: Communication error (status %u)\n", assocID, status);
     fflush(stdout);
   }
@@ -470,8 +481,8 @@ void chargenRestartNotif(unsigned int assocID, void* ulpDataPtr)
 {
     unsigned int length;
     unsigned char buffer[BUFFER_LENGTH];
-   
-    if (verbose) {  
+
+    if (verbose) {
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
@@ -479,10 +490,10 @@ void chargenRestartNotif(unsigned int assocID, void* ulpDataPtr)
     length = 1 + (rand() % 512);
     memset((void *) buffer, 'A', length);
     buffer[length-1] = '\n';
-    
+
     while((!(((struct ulp_data *)ulpDataPtr)->ShutdownReceived)) &&
           (SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                     SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
+                     SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
                      SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS)) {
       if (vverbose) {
           fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
@@ -497,21 +508,21 @@ void chargenRestartNotif(unsigned int assocID, void* ulpDataPtr)
 void echoRestartNotif(unsigned int assocID, void* ulpDataPtr)
 {
     SCTP_AssociationStatus assocStatus;
-    
-    if (verbose) {  
+
+    if (verbose) {
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
-    
+
     SCTP_getAssocStatus(assocID, &assocStatus);
-    
+
     /* update ULP data */
     ((struct ulp_data *) ulpDataPtr)->maximumStreamID = assocStatus.outStreams - 1;
 }
 
 void restartNotif(unsigned int assocID, void* ulpDataPtr)
-{    
-    if (verbose) {  
+{
+    if (verbose) {
         fprintf(stdout, "%-8x: Restart\n", assocID);
         fflush(stdout);
     }
@@ -519,18 +530,18 @@ void restartNotif(unsigned int assocID, void* ulpDataPtr)
 
 void shutdownCompleteNotif(unsigned int assocID, void* ulpDataPtr)
 {
-  if (verbose) {  
+  if (verbose) {
     fprintf(stdout, "%-8x: Shutdown complete\n", assocID);
     fflush(stdout);
   }
   if (ulpDataPtr)
     free((struct ulp_data *) ulpDataPtr);
-    
+
   SCTP_deleteAssociation(assocID);
 }
 
 void queueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, int queueLength, void* ulpDataPtr)
-{	
+{
     if (vverbose) {
         fprintf(stdout, "%-8x: Queue status change notification: Type %d, ID %d, Length %d\n",
                         assocID, queueType, queueID, queueLength);
@@ -539,7 +550,7 @@ void queueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, in
 }
 
 void chargenQueueStatusChangeNotif(unsigned int assocID, int queueType, int queueID, int queueLength, void* ulpDataPtr)
-{	
+{
     unsigned int length;
     unsigned char buffer[BUFFER_LENGTH];
 
@@ -548,15 +559,15 @@ void chargenQueueStatusChangeNotif(unsigned int assocID, int queueType, int queu
                         assocID, queueType, queueID, queueLength);
         fflush(stdout);
     }
-    
+
     if (queueType == SCTP_SEND_QUEUE) {
       length = 1 + (rand() % 512);
       memset((void *)buffer, 'A', length);
       buffer[length-1] = '\n';
-      
+
       while((!(((struct ulp_data *)ulpDataPtr)->ShutdownReceived)) &&
             (SCTP_send(assocID, 0, buffer, length, SCTP_GENERIC_PAYLOAD_PROTOCOL_ID,
-                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive, 
+                       SCTP_USE_PRIMARY, SCTP_NO_CONTEXT, timeToLive,
                        SCTP_ORDERED_DELIVERY, SCTP_BUNDLING_DISABLED) == SCTP_SUCCESS)) {
         if (vverbose) {
             fprintf(stdout, "%-8x: %u bytes sent.\n", assocID, length);
@@ -571,7 +582,7 @@ void chargenQueueStatusChangeNotif(unsigned int assocID, int queueType, int queu
 
 void shutdownReceivedNotif(unsigned int assocID, void* ulpDataPtr)
 {
-    if (verbose) {  
+    if (verbose) {
         fprintf(stdout, "%-8x: Shutdown received\n", assocID);
         fflush(stdout);
     }
@@ -637,7 +648,7 @@ int main(int argc, char **argv)
     /* handle all command line options */
     getArgs(argc, argv);
     checkArgs();
-    
+
     SCTP_initLibrary();
     SCTP_getLibraryParameters(&params);
     params.sendOotbAborts    = sendOOTBAborts;
@@ -649,7 +660,7 @@ int main(int argc, char **argv)
                           MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                           noOfLocalAddresses, localAddressList,
                           echoUlp);
-    
+
     SCTP_registerInstance(DISCARD_PORT,
                           MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                           noOfLocalAddresses, localAddressList,
@@ -664,7 +675,7 @@ int main(int argc, char **argv)
                                          MAXIMUM_NUMBER_OF_IN_STREAMS, MAXIMUM_NUMBER_OF_OUT_STREAMS,
                                          noOfLocalAddresses, localAddressList,
                                          chargenUlp);
-    
+
     SCTP_getAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
     instanceParameters.maxSendQueue = SEND_QUEUE_SIZE;
     SCTP_setAssocDefaults((unsigned short)sctpInstance, &instanceParameters);
@@ -675,6 +686,5 @@ int main(int argc, char **argv)
     }
 
     /* this will never be reached */
-    exit(0);
+    return 0;
 }
-
