@@ -1,21 +1,31 @@
-/***************************************************************************
-                                 parser.c
-                            -------------------
-    begin                : Wed Feb 28 2001
-    copyright            : (C) 2001 by Andreas Lang
-    email                : anla@gmx.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
-
+/*
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
+ *
+ * Copyright (C) 2001 by Andreas Lang
+ *
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Contact: anla@gmx.net
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,7 +34,6 @@
 #include "sctptest.h"
 
 
-
 /**
  * This function reads one command from the script file.
  * @param scriptFile
@@ -270,7 +279,7 @@ void printCommand(struct sctptest_scriptCommand *sc, unsigned int lineNum)
     printf("\n\nLine %u:\n", lineNum);
     printf("COMMAND: %s  with %d params\n", sc->command, sc->numOfParams);
 
-    for (i = 0; i < sc->numOfParams; i++)
+    for (i = 0; i < (int)sc->numOfParams; i++)
         printf("KEY: %s   VALUE: %s\n", sc->param[i].key, sc->param[i].value);
 }
 
