@@ -1,21 +1,31 @@
-/***************************************************************************
-                                 sctptest.h
-                             -------------------
-    begin                : Wed Feb 28 2001
-    copyright            : (C) 2001 by Andreas Lang
-    email                : anla@gmx.net
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
-
+/*
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
+ *
+ * Copyright (C) 2001 by Andreas Lang
+ *
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Contact: anla@gmx.net
+ */
 
 /* payload lengths */
 #define MIN_PAYLOAD_LENGTH   20
@@ -80,9 +90,9 @@ void printCommand(struct sctptest_scriptCommand *, unsigned int);
 
 int processScriptCommand(struct sctptest_scriptCommand *, unsigned int, int);
 
-char *getStrParam(struct sctptest_scriptCommand *, char *, unsigned int *, int, unsigned int);
+char *getStrParam(struct sctptest_scriptCommand *, const char *, unsigned int *, int, unsigned int);
 
-unsigned long getIntParam(struct sctptest_scriptCommand *, char *, unsigned long,
+unsigned long getIntParam(struct sctptest_scriptCommand *, const char *, unsigned long,
                           unsigned long, int, unsigned int *, int, unsigned int);
 
 void doReceive(unsigned int);
