@@ -1,64 +1,46 @@
 /*
- *  $Id: SCTP-control.c 686 2005-08-04 10:49:29Z dreibh $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * Purpose: This module implements SCTP-control.
- *          SCTP-control controls the setup and shutdown of an association.
- *          For this purpose it receives primitives from the ULP (via message-distribution)
- *          and the peer (via (de-)bundling). In response to this input-signals, SCTP-Control
- *          sends control-primitives to the ULP and the peer. It also stores the state of
- *          an association.
- *
- * function prefixes: scu_ for  primitives originating from the ULP
- *                    sctlr_ for primitives originating from the peer
- *                    sci_ for SCTP-internal calls
- *
- * Remarks: Host and network byte order (HBO and NBO):
- *          In this module all data are data are HBO, except the IP-Addresses and message strings.
- *          Thus everything entered into a message string must be converted to NBO and
- *          everthing extracted from a message string must be converted to HBO (except the
- *          IP-addresses).
- *
+ *          andreas.jungmaier@web.de
  */
 
-
-/* TODO : check that in SHUTDOWN states we reset the retransmission counters, any time we
-            get a valid datagram from our peer. This must also be done for SACKs, DATA, etc.
-            we get from peer !
- */
-
-
 #include "SCTP-control.h"
 
 #include "distribution.h"
@@ -158,7 +140,6 @@ static SCTP_controlData *localData;
 static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *unused)
 {
     unsigned int state;
-    boolean initFailed;
     ChunkID shutdownCID;
     ChunkID shutdownAckCID;
     guint primary;
@@ -179,8 +160,6 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
     }
 
     state = localData->association_state;
-    initFailed = FALSE;
-
     primary = pm_readPrimaryPath();
 
     event_logiii(VERBOSE, "sci_timer_expired(AssocID=%u,  state=%u, Primary=%u",
@@ -207,14 +186,13 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
             /* log error to log-file */
             event_log(EXTERNAL_EVENT,
                       "init retransmission counter exeeded threshold in state COOKIE_WAIT");
-            /* report error to ULP tbd: status */
-            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
             /* free memory for initChunk */
             free(localData->initChunk);
             localData->initTimer = 0;
             localData->initChunk = NULL;
-            /* delete this association */
-            initFailed = TRUE;
+            /* report error to ULP tbd: status */
+            mdi_deleteCurrentAssociation();
+            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
         }
         break;
 
@@ -239,14 +217,13 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
             /* log error to log-file */
             event_log(EXTERNAL_EVENT,
                       "init retransmission counter exeeded threshold; state: COOKIE_ECHOED");
-            /* report error to ULP tbd: status */
-            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
             /* free memory for cookieChunk */
             free(localData->cookieChunk);
             localData->initTimer = 0;
             localData->cookieChunk = NULL;
-            /* delete this association */
-            initFailed = TRUE;
+            /* report error to ULP tbd: status */
+            mdi_deleteCurrentAssociation();
+            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
         }
         break;
 
@@ -275,10 +252,13 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
                 adl_startTimer(localData->initTimerDuration, &sci_timer_expired, TIMER_TYPE_SHUTDOWN,
                                 (void *) &localData->associationID, NULL);
         } else {
-            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
+            /* mdi_communicationLostNotif() may call sctp_deleteAssociation().
+               This would invalidate localData and therefore localData->initTimer
+               has to be reset before! */
             localData->initTimer = 0;
             /* shut down failed, delete current association. */
-            initFailed = TRUE;
+            mdi_deleteCurrentAssociation();
+            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
         }
         break;
 
@@ -311,10 +291,12 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
                 adl_startTimer(localData->initTimerDuration, &sci_timer_expired, TIMER_TYPE_SHUTDOWN,
                                 (void *) &localData->associationID, NULL);
         } else {
-            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
+            /* mdi_communicationLostNotif() may call sctp_deleteAssociation().
+               This would invalidate localData and therefore localData->initTimer
+               has to be reset before! */
             localData->initTimer = 0;
-            /* shut down failed, delete current association. */
-            initFailed = TRUE;
+            mdi_deleteCurrentAssociation();
+            mdi_communicationLostNotif(SCTP_COMM_LOST_EXCEEDED_RETRANSMISSIONS);
         }
         break;
 
@@ -326,10 +308,6 @@ static void sci_timer_expired(TimerID timerID, void *associationIDvoid, void *un
     }
 
     localData = NULL;
-    if (initFailed == TRUE) {
-        mdi_deleteCurrentAssociation();
-    }
-
     mdi_clearAssociationData();
 }
 
@@ -696,8 +674,6 @@ int sctlr_init(SCTP_init * init)
     int process_further, result;
     int return_state = STATE_OK;
 
-    guint primary;
-
     event_log(EXTERNAL_EVENT, "sctlr_init() is executed");
 
     initCID = ch_makeChunk((SCTP_simple_chunk *) init);
@@ -817,7 +793,6 @@ int sctlr_init(SCTP_init * init)
 
         state = localData->association_state;
         event_logi(EXTERNAL_EVENT, "sctlr_init: received INIT chunk in state %02u", state);
-        primary = pm_readPrimaryPath();
         supportedTypes = mdi_getSupportedAddressTypes();
 
         switch (state) {
@@ -1316,11 +1291,6 @@ void sctlr_cookie_echo(SCTP_cookie_echo * cookie_echo)
         ch_deleteChunk(initCID);
         ch_deleteChunk(initAckCID);
         event_log(EXTERNAL_EVENT, "event: good cookie echo received, but with incorrect verification tag");
-
-        /* for test purposes */
-        exit(-1);
-        /* for test purposes */
-
         return;
     }
 
@@ -1934,15 +1904,15 @@ int sctlr_shutdownAck()
         }
         break;
     case ESTABLISHED:
-        error_log(ERROR_FATAL,
+        error_log(ERROR_MAJOR,
                   "sctlr_shutdownAck in state ESTABLISHED, peer not standard conform ! ");
         break;
     case SHUTDOWNPENDING:
-        error_log(ERROR_FATAL,
+        error_log(ERROR_MAJOR,
                   "sctlr_shutdownAck in state SHUTDOWNPENDING, peer not standard conform ! ");
         break;
     case SHUTDOWNRECEIVED:
-        error_log(ERROR_FATAL,
+        error_log(ERROR_MAJOR,
                   "sctlr_shutdownAck in state SHUTDOWNRECEIVED, peer not standard conform ! ");
         break;
 
@@ -2312,7 +2282,7 @@ void sci_allChunksAcked()
         if (localData->initTimer != 0) sctp_stopTimer(localData->initTimer);
 
         localData->initTimer =  adl_startTimer(localData->initTimerDuration, &sci_timer_expired,TIMER_TYPE_SHUTDOWN,
-                                (void *) &localData->associationID, NULL);
+                                   (void *) &localData->associationID, NULL);
 
         state = SHUTDOWNACKSENT;
         break;
