@@ -1,59 +1,46 @@
 /*
- *  $Id: chunkHandler.c 704 2005-08-09 16:40:18Z dreibh $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- *
- * Purpose: This module assembles and disassembles control chunks.
- *          When a chunk is created, the caller gets a chunks-ID, with which it can
- *          address the chunk in following calls.
- *          In order to handle more than one chunk at the same time, pointers to chunk
- *          are stored in array of length MAX_CHUNKS. This is required for modules that
- *          receive a chunk and want to respond by sending another chunk.
- *          But a chunk should be deleted after a signal is handled. Else the number of
- *          could not be limited and a list would be required to store them.
- *          As long as the bytestring is not read by ch_chunkString, the length of a chunk is
- *          is stored in host byte order, because it may still change as new elements are
- *          added to the chunk.
- *          After ch_chunkString is called, the chunk should no longer be modified.
- *
- *    TODO :   This module also contains a whole lot of malloc()s, and these are always
- *              freed after the chunks has been dealt with. Why not alloc once, and keep
- *              the memory afterwards ???????? This makes other things more complicated,
- *              though.
- *
- * function prefix: ch_
+ *          andreas.jungmaier@web.de
  */
+
 #ifdef HAVE_CONpm_heartbeatFIG_H
 #include <config.h>
 #endif
@@ -73,7 +60,7 @@
 #define MAX_CHUNKS 8
 
 #ifndef IN_EXPERIMENTAL
-#define  IN_EXPERIMENTAL(a)   ((((long int) (a)) & 0xf0000000) == 0xf0000000)
+#define  IN_EXPERIMENTAL(a)   ((((gint) (a)) & 0xf0000000) == 0xf0000000)
 #endif
 
 #ifndef IN_BADCLASS
@@ -283,7 +270,6 @@ setIPAddresses(unsigned char *mstring, guint16 length, union sockunion addresses
             } else {
                 error_log(ERROR_MAJOR, "parameter problem, abort scanning in setIPAddresses");
                 break;
-
             }
             cursabs += cursrel;
             cursabs += 8;
@@ -306,11 +292,11 @@ setIPAddresses(unsigned char *mstring, guint16 length, union sockunion addresses
         if (localHostFound == FALSE) {
             /* this is from a normal address, get all except loopback */
             if (linkLocalFound) {
-                filterFlags =  flag_Default|flag_HideLoopback;
+                filterFlags = (AddressScopingFlags)(flag_Default|flag_HideLoopback);
             } else if (siteLocalFound) {
-                filterFlags =  flag_Default| flag_HideLinkLocal|flag_HideLoopback;
+                filterFlags = (AddressScopingFlags)(flag_Default|flag_HideLinkLocal|flag_HideLoopback);
             } else {
-                filterFlags = flag_Default|flag_HideLocal;
+                filterFlags = (AddressScopingFlags)(flag_Default|flag_HideLocal);
             }
         } else  /* if localHostFound == TRUE) */ {
              /* this is from a loopback, get all */
@@ -330,7 +316,8 @@ setIPAddresses(unsigned char *mstring, guint16 length, union sockunion addresses
 #ifdef SIN6_LEN
                 tmp_su.sin6.sin6_len = sizeof(struct sockaddr_in6);
 #endif                          /* SIN6_LEN */
-                memcpy(&(tmp_su.sin6.sin6_addr.s6_addr),&(address->dest_addr.sctp_ipv6), sizeof(struct sockaddr_in6));
+                memcpy(&(tmp_su.sin6.sin6_addr.s6_addr),&(address->dest_addr.sctp_ipv6), 16);
+                /* Bugfix: just copy 16 bytes of the address, not sizeof(struct sockaddr_in6)! */
 
 #if defined (LINUX)
                 if (IN6_IS_ADDR_UNSPECIFIED(&(address->dest_addr.sctp_ipv6))) discard = TRUE;
@@ -431,7 +418,7 @@ setIPAddresses(unsigned char *mstring, guint16 length, union sockunion addresses
 }
 
 
-static void enterChunk(SCTP_simple_chunk * chunk, char *log_text)
+static void enterChunk(SCTP_simple_chunk * chunk, const char *log_text)
 {
     unsigned int cid;
 
@@ -812,7 +799,7 @@ int ch_enterPRSCTPfromInit(ChunkID initAckCID, ChunkID initCID)
         pType =  ntohs(vl_initPtr->param_type);
         pLen  =  ntohs(vl_initPtr->param_length);
 
-        if (pLen < 4) result = -1; /* peer error - ignore - should send an error notification */
+        if (pLen < 4) return -1; /* peer error - ignore - should send an error notification */
 
         event_logiii(VERBOSE, "Scan variable parameters: Got type %u, len: %u, position %u",pType, pLen, curs);
 
@@ -1032,7 +1019,7 @@ int ch_enterUnrecognizedParameters(ChunkID initCID, ChunkID AckCID, unsigned int
         ack_string =  &((SCTP_init *)chunks[AckCID])->variableParams[writeCursor[AckCID]];
         vl_initPtr = (SCTP_vlparam_header *) & init_string[curs];
         pType = ntohs(vl_initPtr->param_type);
-        pLen =   ntohs(vl_initPtr->param_length);
+        pLen  = ntohs(vl_initPtr->param_length);
 
         if (pLen < 4)  return -1;
 
@@ -1090,7 +1077,7 @@ int ch_enterUnrecognizedErrors(ChunkID initAckID,
 
     guint16 curs;
     guint16 pType, oType;
-    guint16 pLen, oLen;
+    guint16 pLen;
     guint16 vlp_totalLength;
     gboolean with_ipv4 = FALSE, with_ipv6=FALSE;
     SCTP_ip_address* address = NULL;
@@ -1151,8 +1138,7 @@ int ch_enterUnrecognizedErrors(ChunkID initAckID,
 
         } else if (pType == VLPARAM_UNRECOGNIZED_PARAM) {
             vl_optionsPtr = (SCTP_vlparam_header *) & ack_string[curs+sizeof(SCTP_vlparam_header)];
-            oType =  ntohs(vl_optionsPtr->param_type);
-            oLen =   ntohs(vl_optionsPtr->param_length);
+            oType = ntohs(vl_optionsPtr->param_type);
 
             if (oType ==  VLPARAM_PRSCTP) {
                 *peerSupportsPRSCTP = FALSE;
@@ -1227,7 +1213,7 @@ int ch_enterUnrecognizedErrors(ChunkID initAckID,
 #ifdef HAVE_SIN6_SCOPE_ID
                     preferredDest->sin6.sin6_scope_id = htonl(0);
 #endif
-                    memcpy(preferredDest->sin6.sin6_addr.s6_addr,
+                    memcpy(&preferredDest->sin6.sin6_addr.s6_addr,
                            address->dest_addr.sctp_ipv6, sizeof(struct in6_addr));
                     /* FIXME: check if we got the correct address ! */
                 }
@@ -1943,7 +1929,7 @@ ChunkID ch_makeHeartbeat(unsigned int sendingTime, unsigned int pathID)
     heartbeatChunk->sendingTime = htonl(sendingTime);
 
     key =  key_operation(KEY_READ);
-    if (key == NULL) exit(-111);
+    if (key == NULL) abort();
     memset(heartbeatChunk->hmac, 0, HMAC_LEN);
 
     MD5Init(&ctx);
@@ -1986,7 +1972,7 @@ gboolean ch_verifyHeartbeat(ChunkID chunkID)
     if (chunks[chunkID]->chunk_header.chunk_id == CHUNK_HBACK) {
         heartbeatChunk =  (SCTP_heartbeat *)chunks[chunkID];
         key =  key_operation(KEY_READ);
-        if (key == NULL) exit(-111);
+        if (key == NULL) abort();
         /* store HMAC */
         memcpy(hbSignature, heartbeatChunk->hmac, HMAC_LEN);
 
@@ -2118,7 +2104,7 @@ ch_makeErrorChunk(void)
 
 void
 ch_addUnrecognizedParameter(unsigned char* pos, ChunkID cid,
-                 unsigned short length, unsigned char* data)
+                            unsigned short length, unsigned char* data)
 
 {
     SCTP_error_cause * ec;
@@ -2129,10 +2115,11 @@ ch_addUnrecognizedParameter(unsigned char* pos, ChunkID cid,
     ec = (SCTP_error_cause*) pos;
     ec->cause_code = htons(VLPARAM_UNRECOGNIZED_PARAM);
     ec->cause_length = htons((unsigned short)(length+2*sizeof(unsigned short)));
-    if (length>0) memcpy(ec->cause_information, data, length);
+    if (length > 0) {
+        memcpy(&ec->cause_information, data, length);
+    }
     writeCursor[cid] += (length + 2*sizeof(unsigned short));
     while ((writeCursor[cid] % 4) != 0) writeCursor[cid]++;
-
 }
 
 
@@ -2140,7 +2127,7 @@ ch_addUnrecognizedParameter(unsigned char* pos, ChunkID cid,
 
 void
 ch_addParameterToInitChunk(ChunkID initChunkID, unsigned short pCode,
-                       unsigned short dataLength, unsigned char* data)
+                           unsigned short dataLength, unsigned char* data)
 {
     SCTP_UnrecognizedParams *vlPtr = NULL;
     unsigned short index;
@@ -2188,6 +2175,9 @@ ch_enterErrorCauseData(ChunkID chunkID, unsigned short code,
     ec = (SCTP_error_cause*) &(chunks[chunkID]->simple_chunk_data[index]);
     ec->cause_code = htons(code);
     ec->cause_length = htons((unsigned short)(length+2*sizeof(unsigned short)));
+    if (length > 0) {
+        memcpy(&ec->cause_information, data, length);
+    }
     writeCursor[chunkID] += (length + 2*sizeof(unsigned short));
     while ((writeCursor[chunkID] % 4) != 0) writeCursor[chunkID]++;
 
