@@ -1,47 +1,46 @@
 /*
- * $Id: flowcontrol.c 686 2005-08-04 10:49:29Z dreibh $
- * SCTP implementation according to RFC 2960.
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
+ *
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * This module implements most parts of the flow control mechanisms
- *
+ *          andreas.jungmaier@web.de
  */
 
-
 #include "flowcontrol.h"
 #include "bundling.h"
 #include "adaptation.h"
@@ -136,7 +135,7 @@ void *fc_new_flowcontrol(unsigned int peer_rwnd,
     fc_data *tmp;
     unsigned int count;
 
-    tmp = malloc(sizeof(fc_data));
+    tmp = (fc_data*)malloc(sizeof(fc_data));
     if (!tmp)
         error_log(ERROR_FATAL, "Malloc failed");
     tmp->current_tsn = my_iTSN;
@@ -145,15 +144,15 @@ void *fc_new_flowcontrol(unsigned int peer_rwnd,
                "Flowcontrol: ===== Num of number_of_destination_addresses = %d ",
                number_of_destination_addresses);
 
-    tmp->cparams = malloc(number_of_destination_addresses * sizeof(cparm));
+    tmp->cparams = (cparm*)malloc(number_of_destination_addresses * sizeof(cparm));
     if (!tmp->cparams)
         error_log(ERROR_FATAL, "Malloc failed");
 
-    tmp->T3_timer = malloc(number_of_destination_addresses * sizeof(TimerID));
+    tmp->T3_timer = (TimerID*)malloc(number_of_destination_addresses * sizeof(TimerID));
     if (!tmp->T3_timer)
         error_log(ERROR_FATAL, "Malloc failed");
 
-    tmp->addresses = malloc(number_of_destination_addresses * sizeof(unsigned int));
+    tmp->addresses = (unsigned int*)malloc(number_of_destination_addresses * sizeof(unsigned int));
     if (!tmp->addresses)
         error_log(ERROR_FATAL, "Malloc failed");
 
@@ -500,7 +499,7 @@ void fc_timer_cb_t3_timeout(TimerID tid, void *assoc, void *data2)
         return;
     }
 
-    chunks = malloc(num_of_chunks * sizeof(chunk_data *));
+    chunks = (chunk_data**)malloc(num_of_chunks * sizeof(chunk_data *));
     num_of_chunks = rtx_t3_timeout(&(fc->my_association), ad_idx, fc->cparams[ad_idx].mtu, chunks);
     if (num_of_chunks <= 0) {
         event_log(VERBOSE, "No Chunks to re-transmit - AFTER calling rtx_t3_timeout - returning");
@@ -595,7 +594,7 @@ void fc_update_chunk_data(fc_data * fc, chunk_data * dat, unsigned int destinati
         dat->num_of_transmissions = MAX_DEST - 1;
     } else if (dat->num_of_transmissions < 1) {
         error_log(ERROR_FATAL, "Somehow dat->num_of_transmissions became 0 !");
-        exit(-1);
+        abort();
     }
 
     /* this time we will send dat to destination */
@@ -659,11 +658,10 @@ gboolean fc_send_okay(fc_data* fc,
  */
 int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doInitialRetransmit)
 {
-    int result;
     unsigned int len, obpa;
     fc_data *fc;
     chunk_data *dat;
-    unsigned int total_size, destination, oldDestination, peer_rwnd, rto_time;
+    unsigned int total_size, destination, oldDestination, peer_rwnd;
 
     gboolean data_is_retransmitted = FALSE;
     gboolean lowest_tsn_is_retransmitted = FALSE;
@@ -675,7 +673,7 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
     fc = (fc_data *) fc_instance;
 
     if (fc->chunk_list != NULL) {
-        dat = g_list_nth_data(fc->chunk_list, 0);
+        dat = (chunk_data*)g_list_nth_data(fc->chunk_list, 0);
     } else {
         return -1;
     }
@@ -739,7 +737,7 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
                     dat->ack_time, dat->num_of_transmissions);
         /* -------------------- DEBUGGING --------------------------------------- */
 
-        result = bu_put_Data_Chunk((SCTP_simple_chunk *) dat->data, &destination);
+        bu_put_Data_Chunk((SCTP_simple_chunk *) dat->data, &destination);
         data_is_submitted = TRUE;
         adl_gettime(&(fc->cparams[destination].last_send_time));
 
@@ -752,7 +750,7 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
         if (dat->num_of_transmissions == 1) {
             adl_gettime(&(dat->transmission_time));
             event_log(INTERNAL_EVENT_0, "Storing chunk in retransmission list -> calling rtx_save_retrans");
-            result = rtx_save_retrans_chunks(dat);
+            rtx_save_retrans_chunks(dat);
         } else {
             if (lowest_tsn_is_retransmitted == FALSE)
                 /* must not be reset to FALSE here */
@@ -762,7 +760,7 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
         fc->chunk_list = g_list_remove(fc->chunk_list, (gpointer) dat);
         fc->list_length--;
 
-        dat = g_list_nth_data(fc->chunk_list, 0);
+        dat = (chunk_data*)g_list_nth_data(fc->chunk_list, 0);
         if (dat != NULL) {
             if (dat->num_of_transmissions >= 1)    data_is_retransmitted = TRUE;
             else if (dat->num_of_transmissions == 0) data_is_retransmitted = FALSE;
@@ -829,7 +827,6 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
     if (data_is_submitted == TRUE) {
         fc->one_packet_inflight = TRUE;
         bu_sendAllChunks(&destination);
-        rto_time = pm_readRTO((short)destination);
 
         if (fc->maxQueueLen != 0) {
             if (len < fc->maxQueueLen && oldListLen >= fc->maxQueueLen) {
@@ -845,7 +842,7 @@ int fc_check_for_txmit(void *fc_instance, unsigned int oldListLen, gboolean doIn
         if (len < fc->maxQueueLen && oldListLen >= fc->maxQueueLen) {
             mdi_queueStatusChangeNotif(SCTP_SEND_QUEUE, 0, len);
         } else if (len > fc->maxQueueLen && oldListLen <= fc->maxQueueLen) {
-             mdi_queueStatusChangeNotif(SCTP_SEND_QUEUE, 0, len);
+            mdi_queueStatusChangeNotif(SCTP_SEND_QUEUE, 0, len);
         }
     }
 
@@ -1279,6 +1276,17 @@ void fc_sack_info(unsigned int address_index, unsigned int arwnd,unsigned int ct
     if (fc->chunk_list != NULL) {
         fc_check_for_txmit(fc, oldListLen, FALSE);
     }
+
+    else {
+        if (fc->maxQueueLen != 0) {
+            /* Bugfix: Without this, no queueStatusChangeNotif would be
+                       called if there are fragmented chunks! */
+            if (fc->outstanding_bytes == 0) {
+                mdi_queueStatusChangeNotif(SCTP_SEND_QUEUE, 0, fc->list_length);
+            }
+        }
+    }
+
     return;
 }    /* end: fc_sack_info  */
 
@@ -1295,7 +1303,7 @@ int fc_dequeueUnackedChunk(unsigned int tsn)
         error_log(ERROR_MAJOR, "flow control instance not set !");
         return SCTP_MODULE_NOT_FOUND;
     }
-    dat = g_list_nth_data(fc->chunk_list, 0);
+    dat = (chunk_data*)g_list_nth_data(fc->chunk_list, 0);
     tmp = fc->chunk_list;
     while (dat != NULL && tmp != NULL) {
         event_logii(VVERBOSE, "fc_dequeueOldestUnsentChunks(): checking chunk tsn=%u, num_rtx=%u ", dat->chunk_tsn, dat->num_of_transmissions);
@@ -1320,8 +1328,6 @@ int fc_dequeueUnackedChunk(unsigned int tsn)
     }
     /* else */
     return 0;
-
-
 }
 
 int fc_dequeueOldestUnsentChunk(unsigned char *buf, unsigned int *len, unsigned int *tsn,
@@ -1344,7 +1350,7 @@ int fc_dequeueOldestUnsentChunk(unsigned char *buf, unsigned int *len, unsigned
 
     if (listlen <= 0)               return SCTP_UNSPECIFIED_ERROR;
     if (fc->chunk_list == NULL) return  SCTP_UNSPECIFIED_ERROR;
-    dat = g_list_nth_data(fc->chunk_list, 0);
+    dat = (chunk_data*)g_list_nth_data(fc->chunk_list, 0);
     tmp = fc->chunk_list;
     while (dat != NULL && tmp != NULL) {
         event_logii(VVERBOSE, "fc_dequeueOldestUnsentChunks(): checking chunk tsn=%u, num_rtx=%u ", dat->chunk_tsn, dat->num_of_transmissions);
