@@ -1,44 +1,46 @@
 /*
- *  $Id: pathmanagement.c 532 2004-11-17 21:11:53Z tuexen $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * Purpose: This module does faultmanagement for paths.
- *
- * function prefix: pm_
+ *          andreas.jungmaier@web.de
  */
 
-
 #include "globals.h"
 #include "chunkHandler.h"
 #include "SCTP-control.h"
@@ -46,7 +48,6 @@
 #include "bundling.h"
 #include "pathmanagement.h"
 
-
 /*------------------------ defines -----------------------------------------------------------*/
 #define RTO_ALPHA            0.125
 #define RTO_BETA              0.25
@@ -184,14 +185,14 @@ static gboolean handleChunksRetransmitted(short pathID)
                  pathID, pmData->pathData[pathID].pathRetranscount, pmData->peerRetranscount);
 
     if (pmData->pathData[pathID].state == PM_PATH_UNCONFIRMED) {
-    
+
         pmData->pathData[pathID].pathRetranscount++;
-        
+
     } else if (pmData->pathData[pathID].state == PM_ACTIVE) {
-    
+
         pmData->pathData[pathID].pathRetranscount++;
         pmData->peerRetranscount++;
-        
+
     } else {
         event_log(INTERNAL_EVENT_0,
                   "handleChunksRetransmitted: ignored, because already inactive");
@@ -268,7 +269,7 @@ static void handleChunksAcked(short pathID, unsigned int newRTT)
         } else {
             pmData->pathData[pathID].rttvar = (unsigned int)
                 ((1. - RTO_BETA) * pmData->pathData[pathID].rttvar +
-                RTO_BETA * abs(pmData->pathData[pathID].srtt - newRTT));
+                RTO_BETA * abs((int)pmData->pathData[pathID].srtt - (int)newRTT));
             pmData->pathData[pathID].rttvar = max((unsigned int)pmData->pathData[pathID].rttvar, GRANULARITY);
 
             pmData->pathData[pathID].srtt = (unsigned int)
@@ -497,14 +498,14 @@ void pm_heartbeatAck(SCTP_heartbeat * heartbeatChunk)
         error_log(ERROR_FATAL, "pm_heartbeatAck: FALSE SIGNATURE !!!!!!!!!!!!!!!");
         return;
     }
-    
+
     ch_forgetChunk(heartbeatCID);
 
     if (!(pathID >= 0 && pathID < pmData->numberOfPaths)) {
         error_logi(ERROR_MAJOR, "pm_heartbeatAck: invalid path ID %d", pathID);
         return;
     }
-    
+
     /* this also resets error counters */
     handleChunksAcked(pathID, roundtripTime);
 
@@ -827,6 +828,7 @@ int pm_disableHB(short pathID)
 
     if (pmData->pathData[pathID].heartbeatEnabled) {
         sctp_stopTimer(pmData->pathData[pathID].hearbeatTimer);
+        pmData->pathData[pathID].hearbeatTimer = 0;
         pmData->pathData[pathID].heartbeatEnabled = FALSE;
         event_logi(INTERNAL_EVENT_0, "pm_disableHB: path %d disabled", (unsigned int) pathID);
     }
@@ -1119,7 +1121,7 @@ int  pm_getRtoMax(void)
 int  pm_setHBInterval(unsigned int new_interval)
 {
     int count;
-    
+
     pmData = (PathmanData *) mdi_readPathMan();
 
     if (pmData == NULL) {
@@ -1215,7 +1217,7 @@ short pm_setPaths(short noOfPaths, short primaryPathID)
             pmData->pathData[i].pathID = i;
 
             b = mdi_getDefaultMaxBurst();
-            
+
             if (i != primaryPathID) {
                 j++;
                 if (j < b) {
@@ -1240,7 +1242,7 @@ short pm_setPaths(short noOfPaths, short primaryPathID)
                                     TIMER_TYPE_HEARTBEAT,
                                     (void *) &pmData->associationID,
                                     (void *) &pmData->pathData[i].pathID);
-            }                
+            }
             /* after RTO we can do next RTO update */
             adl_gettime(&(pmData->pathData[i].rto_update));
 
@@ -1302,8 +1304,10 @@ void pm_deletePathman(void *pathmanPtr)
 
     if (pmData != NULL && pmData->pathData != NULL) {
         for (i = 0; i < pmData->numberOfPaths; i++) {
-            if (pmData->pathData[i].state == PM_ACTIVE)
-                sctp_stopTimer(pmData->pathData[i].hearbeatTimer);
+            if (pmData->pathData[i].hearbeatTimer != 0) {
+                adl_stopTimer(pmData->pathData[i].hearbeatTimer);
+                pmData->pathData[i].hearbeatTimer = 0;
+            }
         }
     }
 
