@@ -1,41 +1,44 @@
 /*
- *  $Id: recvctrl.c 670 2005-07-28 15:53:38Z dreibh $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * This module creates SACK data structures, that may be used to
- * indicate received and lost data to the association peer.
- *
+ *          andreas.jungmaier@web.de
  */
 
 #include "recvctrl.h"
@@ -104,7 +107,7 @@ void *rxc_new_recvctrl(unsigned int remote_initial_TSN, unsigned int number_of_d
 /*
     unsigned int count;
 */
-    tmp = malloc(sizeof(rxc_buffer));
+    tmp = (rxc_buffer*)malloc(sizeof(rxc_buffer));
     if (!tmp) error_log(ERROR_FATAL, "Malloc failed");
 
     tmp->frag_list = NULL;
@@ -270,7 +273,7 @@ void rxc_update_duplicates(rxc_buffer * rbuf, unsigned int ch_tsn)
         current = g_list_next(current);
     }
     /* its new - add it to the list */
-    match = malloc(sizeof(duplicate));
+    match = (duplicate*)malloc(sizeof(duplicate));
     match->duplicate_tsn = ch_tsn;
     rbuf->dup_list =  g_list_insert_sorted(rbuf->dup_list, match, (GCompareFunc) rxc_sort_duplicates);
 
@@ -357,7 +360,7 @@ boolean rxc_update_fragments(rxc_buffer * rbuf, unsigned int ch_tsn)
                     rbuf->new_chunk_received = TRUE;
                     return TRUE;
                 } else {    /* a fragment in between */
-                    new_frag = malloc(sizeof(fragment32));
+                    new_frag = (fragment32*)malloc(sizeof(fragment32));
                     new_frag->start_tsn = new_frag->stop_tsn = ch_tsn;
                     event_log(VVERBOSE, "Inserting new fragment....");
                     rbuf->frag_list = g_list_insert_sorted(rbuf->frag_list, new_frag, (GCompareFunc) rxc_sort_fragments);
@@ -420,7 +423,7 @@ boolean rxc_update_fragments(rxc_buffer * rbuf, unsigned int ch_tsn)
         return FALSE;
     } else {                    /* a new fragment altogether */
         current = g_list_last(rbuf->frag_list);
-        new_frag = malloc(sizeof(fragment32));
+        new_frag = (fragment32*)malloc(sizeof(fragment32));
         new_frag->start_tsn = ch_tsn;
         new_frag->stop_tsn = ch_tsn;
         rbuf->frag_list = g_list_append(rbuf->frag_list, new_frag);
@@ -511,9 +514,13 @@ int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
 
     event_logi(VVERBOSE, "rxc_data_chunk_rx: after rxc_bubbleup_ctsna, rxc->ctsna=%u", rxc->ctsna);
 
-    if (rxc->new_chunk_received == TRUE) se_recvDataChunk(se_chk, chunk_len, ad_idx);
-    /* resetting it */
-    rxc->new_chunk_received = FALSE;
+    if (rxc->new_chunk_received == TRUE) {
+        if(se_recvDataChunk(se_chk, chunk_len, ad_idx) == SCTP_SUCCESS) {
+            /* resetting it */
+            rxc->new_chunk_received = FALSE;
+        }
+        /* else: ABORT has been sent and the association (possibly) removed in callback! */
+    }
     return 1;
 }
 
@@ -525,7 +532,6 @@ int rxc_data_chunk_rx(SCTP_data_chunk * se_chk, unsigned int ad_idx)
 boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
 {
     rxc_buffer *rxc;
-    int result;
     unsigned int num_of_frags;
 
     event_logii(VVERBOSE,
@@ -556,7 +562,7 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
     /* first sack is sent at once, since datagrams_received==-1 */
     if (force_sack == TRUE) {
         rxc->lowest = rxc->ctsna;
-        result = bu_put_SACK_Chunk(rxc->sack_chunk, destination_address);
+        bu_put_SACK_Chunk((SCTP_sack_chunk*)rxc->sack_chunk, destination_address);
         return TRUE;
     } else {
 
@@ -569,7 +575,7 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
                 return FALSE;
         }
         rxc->lowest = rxc->ctsna;
-        result = bu_put_SACK_Chunk(rxc->sack_chunk,destination_address);
+        bu_put_SACK_Chunk((SCTP_sack_chunk*)rxc->sack_chunk,destination_address);
         return TRUE;
     }
     return FALSE;
@@ -588,7 +594,6 @@ boolean rxc_create_sack(unsigned int *destination_address, boolean force_sack)
 void rxc_sack_timer_cb(TimerID tid, void *assoc, void *dummy)
 {
     unsigned short res;
-    int send_result;
 
     rxc_buffer *rxc;
     event_log(INTERNAL_EVENT_1, "Timer Callback Function activated -> initiate sending of a SACK");
@@ -609,7 +614,7 @@ void rxc_sack_timer_cb(TimerID tid, void *assoc, void *dummy)
     /* sending sack */
     /* FIXME : maybe choose different address ??? */
     rxc_create_sack(&rxc->last_address, TRUE);
-    send_result = bu_sendAllChunks(&rxc->last_address);
+    bu_sendAllChunks(&rxc->last_address);
 
     mdi_clearAssociationData();
     return;
@@ -660,7 +665,6 @@ boolean rxc_sack_timer_is_running(void)
  */
 void rxc_all_chunks_processed(boolean new_data_received)
 {
-
     /* now go and create SACK structure from the array */
     rxc_buffer *rxc=NULL;
     SCTP_sack_chunk *sack=NULL;
@@ -705,7 +709,7 @@ void rxc_all_chunks_processed(boolean new_data_received)
     if (current_rwnd > 0 && current_rwnd <= 2 * MAX_SCTP_PDU) current_rwnd = 1;
 
 
-    sack = rxc->sack_chunk;
+    sack = (SCTP_sack_chunk*)rxc->sack_chunk;
     sack->chunk_header.chunk_id = CHUNK_SACK;
     sack->chunk_header.chunk_flags = 0;
     len16 = sizeof(SCTP_chunk_header) + (2 + num_of_dups) * sizeof(unsigned int) +
@@ -770,7 +774,7 @@ void rxc_all_chunks_processed(boolean new_data_received)
 int rxc_start_sack_timer(unsigned int oldQueueLen)
 {
     rxc_buffer *rxc;
-    int bytesQueued = 0, send_result;
+    int bytesQueued = 0;
 
     rxc = (rxc_buffer *) mdi_readRX_control();
     if (!rxc) {
@@ -786,7 +790,7 @@ int rxc_start_sack_timer(unsigned int oldQueueLen)
         (rxc->my_rwnd - bytesQueued >= 2 * MAX_SCTP_PDU)) {
         /* send SACK at once */
         rxc_create_sack(&rxc->last_address, TRUE);
-        send_result = bu_sendAllChunks(&rxc->last_address);
+        bu_sendAllChunks(&rxc->last_address);
         rxc_stop_sack_timer();
     } else {    /* normal application read, no need to rush things */
         if (rxc->timer_running != TRUE) {
@@ -1010,7 +1014,7 @@ int rxc_process_forward_tsn(void* chunk)
                 rxc->ctsna = fw_tsn;
             } else {
                 error_log(ERROR_FATAL, "rxc_process_forward_tsn: impossible conditon");
-                exit(-1);
+                abort();
             }
         }
         /* we are still here, take next fragment == first fragment */
