@@ -1,41 +1,44 @@
 /*
- *  $Id: reltransfer.c 686 2005-08-04 10:49:29Z dreibh $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * This module implements the retransmission mechanism, and stores
- * all data that has not been acknowledged by the peer.
- *
+ *          andreas.jungmaier@web.de
  */
 
 #include "adaptation.h"
@@ -127,7 +130,7 @@ void *rtx_new_reltransfer(unsigned int number_of_destination_addresses, unsigned
 {
     rtx_buffer *tmp;
 
-    tmp = malloc(sizeof(rtx_buffer));
+    tmp = (rtx_buffer*)malloc(sizeof(rtx_buffer));
     if (!tmp)
         error_log(ERROR_FATAL, "Malloc failed");
 
@@ -298,7 +301,7 @@ int rtx_dequeue_up_to(unsigned int ctsna, unsigned int addr_index)
     tmp = g_list_first(rtx->chunk_list);
 
     while (tmp) {
-        dat = g_list_nth_data(rtx->chunk_list, 0);
+        dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, 0);
         if (!dat) return -1;
 
         chunk_tsn = dat->chunk_tsn;
@@ -399,7 +402,7 @@ static int rtx_advancePeerAckPoint(rtx_buffer *rtx)
     tmp = g_list_first(rtx->chunk_list);
 
     while (tmp) {
-        dat = g_list_nth_data(tmp, 0);
+        dat = (chunk_data*)g_list_nth_data(tmp, 0);
         if (!dat) return -1;
         if (!dat->hasBeenDropped) return 0;
         event_logi(VVERBOSE, "rtx_advancePeerAckPoint: Set advancedPeerAckPoint to %u", dat->chunk_tsn);
@@ -461,7 +464,7 @@ int rtx_get_obpa(unsigned int adIndex, unsigned int *totalInFlight)
         return 0;
     }
     for (count = 0; count < len; count++) {
-        dat = g_list_nth_data(rtx->chunk_list, count);
+        dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, count);
         if (dat == NULL) break;
         /* do not count chunks that were retransmitted by T3 timer              */
         /* dat->hasBeenRequeued will be set to FALSE when these are sent again  */
@@ -579,7 +582,7 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
         } else {
             /* this may become expensive !!!!!!!!!!!!!!!! */
             pos = 0;
-            dat = g_list_nth_data(rtx->chunk_list, i);
+            dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, i);
             if (rtx->chunk_list != NULL && dat != NULL) {
                 do {
                 frag = (fragment *) & (sack->fragments_and_dups[pos]);
@@ -624,7 +627,7 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
                         }     /*  if (dat->gap_reports == 4) */
                         /* read next chunk */
                         i++;
-                        dat = g_list_nth_data(rtx->chunk_list, i);
+                        dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, i);
                         if (dat == NULL)
                             break; /* was the last chunk in the list */
                         if (chunks_to_rtx == MAX_NUM_OF_CHUNKS)
@@ -656,7 +659,7 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
                         dat->gap_reports = 0;
 
                         i++;
-                        dat = g_list_nth_data(rtx->chunk_list, i);
+                        dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, i);
                         if (dat == NULL)
                             break; /* was the last chunk in the list or chunk is empty*/
                         else continue;
@@ -680,11 +683,11 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
         if (rtx->all_chunks_are_unacked == FALSE) {
             /* okay, we have chunks in the queue that were acked by a gap report before       */
             /* and reneged: reset their status to unacked, since that is what peer reported   */
-            /* fast retransmit reneged chunks, as per section   6.2.1.D.iii) of RFC 2960      */
+            /* fast retransmit reneged chunks, as per section   6.2.1.D.iii) of RFC 4960      */
             event_log(VVERBOSE, "rtx_process_sack: resetting all *hasBeenAcked* attributes");
             tmp_list = g_list_first(rtx->chunk_list);
             while (tmp_list) {
-                dat = g_list_nth_data(tmp_list, 0);
+                dat = (chunk_data*)g_list_nth_data(tmp_list, 0);
                 if (!dat) break;
                 if (dat->hasBeenAcked == TRUE && dat->hasBeenDropped == FALSE) {
                     dat->hasBeenAcked = FALSE;
@@ -714,7 +717,7 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
      * also see section 6.2.1 (Note)
      */
     if (rtx->chunk_list == NULL) {
-        if ((rtx->highest_tsn == rtx->highest_acked)) {
+        if (rtx->highest_tsn == rtx->highest_acked) {
             all_acked = TRUE;
         }
         /* section 6.2.1.D.ii) */
@@ -731,7 +734,7 @@ int rtx_process_sack(unsigned int adr_index, void *sack_chunk, unsigned int tota
     } else {
         /* there are still chunks in that queue */
         if (rtx->chunk_list != NULL)
-            dat = g_list_nth_data(rtx->chunk_list, 0);
+            dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, 0);
         if (dat == NULL) {
             error_log(ERROR_FATAL, "Problem with RTX-chunklist, CHECK Program and List Handling");
             return -1;
@@ -828,7 +831,7 @@ int rtx_t3_timeout(void *assoc_id, unsigned int address, unsigned int mtu, chunk
                         /* chunk has expired, maybe send FORWARD_TSN */
                         ((chunk_data *)(tmp->data))->hasBeenDropped = TRUE;
                     } else { /* chunk has not yet expired */
-                        chunks[chunks_to_rtx] = tmp->data;
+                        chunks[chunks_to_rtx] = (chunk_data*)tmp->data;
                         size += chunks[chunks_to_rtx]->chunk_len;
                         event_logii(VVERBOSE, "Scheduling chunk (tsn==%u), len==%u for rtx",
                                     chunks[chunks_to_rtx]->chunk_tsn, chunks[chunks_to_rtx]->chunk_len);
@@ -837,7 +840,7 @@ int rtx_t3_timeout(void *assoc_id, unsigned int address, unsigned int mtu, chunk
                         chunks_to_rtx++;
                     }
                 } else {
-                    chunks[chunks_to_rtx] = tmp->data;
+                    chunks[chunks_to_rtx] = (chunk_data*)tmp->data;
                     size += chunks[chunks_to_rtx]->chunk_len;
                     event_logii(VVERBOSE, "Scheduling chunk (tsn==%u), len==%u for rtx",
                             chunks[chunks_to_rtx]->chunk_tsn, chunks[chunks_to_rtx]->chunk_len);
@@ -853,7 +856,7 @@ int rtx_t3_timeout(void *assoc_id, unsigned int address, unsigned int mtu, chunk
     event_logi(VVERBOSE, "Scheduled %d chunks for rtx", chunks_to_rtx);
 
     if (rtx->chunk_list != NULL) {
-        dat = g_list_nth_data(rtx->chunk_list, 0);
+        dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, 0);
         if (dat == NULL) {
            error_log(ERROR_FATAL, "Problem with RTX-chunklist, CHECK Program and List Handling");
            return chunks_to_rtx;
@@ -934,12 +937,12 @@ void chunk_list_debug(short event_log_level, GList * chunk_list)
             event_log(event_log_level, " Size of List == 0 ! ");
         } else if (size <= 200) {
             event_logi(event_log_level, " Size of List == %u ! Printing first 10 chunks....", size);
-            dat = g_list_nth_data(chunk_list, 0);
+            dat = (chunk_data*)g_list_nth_data(chunk_list, 0);
             last_tsn = dat->chunk_tsn - 1;
             if (size > 10) counter = 10;
             else counter = size;
             for (i=0; i<counter; i++) {
-                dat = g_list_nth_data(chunk_list, i);
+                dat = (chunk_data*)g_list_nth_data(chunk_list, i);
                 event_logii(event_log_level,
                             "________________ Chunk _________________\nChunk Size %u  -- TSN : %u  ",
                             dat->chunk_len, dat->chunk_tsn);
@@ -957,7 +960,7 @@ void chunk_list_debug(short event_log_level, GList * chunk_list)
                 last_tsn = dat->chunk_tsn;
             }
             for (i=counter; i<size; i++) {
-                dat = g_list_nth_data(chunk_list, i);
+                dat = (chunk_data*)g_list_nth_data(chunk_list, i);
                 if (! after(dat->chunk_tsn, last_tsn))
                     error_log(ERROR_FATAL, "Higher TSNs not in sequence ! Terminate");
                 if (dat->chunk_tsn - last_tsn > 10000)
@@ -1053,7 +1056,7 @@ int rtx_dequeueOldestUnackedChunk(unsigned char *buf, unsigned int *len, unsigne
     if (rtx->chunk_list == NULL) return  SCTP_UNSPECIFIED_ERROR;
     listlen = g_list_length(rtx->chunk_list);
     if (listlen <= 0) return SCTP_UNSPECIFIED_ERROR;
-    dat = g_list_nth_data(rtx->chunk_list, 0);
+    dat = (chunk_data*)g_list_nth_data(rtx->chunk_list, 0);
     if (dat->num_of_transmissions == 0) return SCTP_UNSPECIFIED_ERROR;
     if ((*len) <  (dat->chunk_len - FIXED_DATA_CHUNK_SIZE)) return SCTP_BUFFER_TOO_SMALL;
 
