@@ -1,44 +1,44 @@
 /*
- *  $Id: sbundling.c 193 2003-09-10 21:34:40Z tuexen $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * Purpose: This module bundels chunks chunks to be send into UDP-datagramms. Chunks are accepted
- *          with the putChunk function until sendChunk is called, which causes the transmission of
- *          all chunks accumulated so far.
- *          In the receive direction, this module gets UDP-dataparts from SCTP message-distribution,
- *          which are disassambled (debundled) into chunks. Depending on the chunk-type, the chunk
- *          are distributed to SCTP-control, RX-control, pathmanagement or reliable transfer.
+ *          andreas.jungmaier@web.de
  */
 
 #include <stdio.h>
@@ -101,7 +101,7 @@ static bundling_instance *global_buffer;
 
 void bu_init_bundling(void)
 {
-    global_buffer = bu_new();
+    global_buffer = (bundling_instance*)bu_new();
 }
 
 /**
@@ -113,7 +113,7 @@ gpointer bu_new(void)
     /* Alloc new bundling_instance data struct */
     bundling_instance *ptr;
 
-    ptr = malloc(sizeof(bundling_instance));
+    ptr = (bundling_instance*)malloc(sizeof(bundling_instance));
     if (!ptr) {
         error_log(ERROR_MAJOR, "Malloc failed");
         return 0;
@@ -192,7 +192,6 @@ void bu_unlock_sender(guint* ad_idx)
  */
 gint bu_put_SACK_Chunk(SCTP_sack_chunk * chunk, unsigned int * dest_index)
 {
-    gint result;
     bundling_instance *bu_ptr;
     gboolean lock;
 
@@ -211,7 +210,7 @@ gint bu_put_SACK_Chunk(SCTP_sack_chunk * chunk, unsigned int * dest_index)
                   "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
                     (dest_index==NULL)?0:*dest_index);
         if (lock) bu_ptr->locked = FALSE;
-        result = bu_sendAllChunks(dest_index);
+        bu_sendAllChunks(dest_index);
         if (lock) bu_ptr->locked = TRUE;
     } else if (dest_index != NULL) {
         bu_ptr->got_send_address = TRUE;
@@ -245,7 +244,6 @@ gint bu_put_SACK_Chunk(SCTP_sack_chunk * chunk, unsigned int * dest_index)
  */
 gint bu_put_Ctrl_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
 {
-    gint result;
     bundling_instance *bu_ptr;
     gint count;
     gboolean lock;
@@ -265,7 +263,7 @@ gint bu_put_Ctrl_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
                   "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
                     (dest_index==NULL)?0:*dest_index);
         if (lock) bu_ptr->locked = FALSE;
-        result = bu_sendAllChunks(dest_index);
+        bu_sendAllChunks(dest_index);
         if (lock) bu_ptr->locked = TRUE;
     } else if (dest_index != NULL) {
         bu_ptr->got_send_address = TRUE;
@@ -311,7 +309,6 @@ gboolean bu_userDataOutbound(void)
  */
 gint bu_put_Data_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
 {
-    gint result;
     bundling_instance *bu_ptr;
     gint count;
     gboolean lock;
@@ -331,7 +328,7 @@ gint bu_put_Data_Chunk(SCTP_simple_chunk * chunk,unsigned int * dest_index)
                   "Chunk Length exceeded MAX_SCTP_PDU : sending chunk to address %u !",
                     (dest_index==NULL)?0:*dest_index);
         if (lock) bu_ptr->locked = FALSE;
-        result = bu_sendAllChunks(dest_index);
+        bu_sendAllChunks(dest_index);
         if (lock) bu_ptr->locked = TRUE;
     } else if (dest_index != NULL) {
         bu_ptr->got_send_address = TRUE;
@@ -461,7 +458,7 @@ gint bu_sendAllChunks(guint * ad_idx)
         fprintf(stderr, "MTU definitely exceeded (%u) - aborting\n",send_len);
         fprintf(stderr, "sack_position: %u, ctrl_position: %u, data_position: %u\n",
             bu_ptr->sack_position,bu_ptr->ctrl_position,bu_ptr->data_position);
-        exit(-1);
+        abort();
     }
 
     if ((bu_ptr->data_in_buffer) && (idx != -1)) pm_chunksSentOn(idx);
