@@ -1,45 +1,44 @@
 /*
- *  $Id: sctp.h 629 2005-04-29 19:54:18Z tuexen $
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
  *
- * SCTP implementation according to RFC 2960.
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * There are two mailinglists available at www.sctp.de which should be used for
- * any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- * Purpose: This header-file defines the API to the SCTP library, and contains
- * 			all functions and structures the library offers to an application.
- *          Further comments on these functions can be found in the implemen-
- *          tation of this interface in the files distribution.c, adaptation.c
- *          and in the accompanying manual (see subdirectory manual)
- *
- *  function prefix: sctp_
+ *          andreas.jungmaier@web.de
  */
 
 #ifndef SCTP_H
@@ -67,7 +66,7 @@ extern "C" {
 
 #define SCTP_MAJOR_VERSION      1
 #define SCTP_MINOR_VERSION      0
-#define SCTP_TINY_VERSION       5
+#define SCTP_TINY_VERSION       8
 
 /* the maximum length of an IP address string (IPv4 or IPv6, NULL terminated) */
 /* see RFC 1884 (mixed IPv6/Ipv4 addresses)   */
@@ -142,6 +141,7 @@ extern "C" {
 #define SCTP_WRONG_STATE                    -13
 #define SCTP_BUFFER_TOO_SMALL               -14
 #define SCTP_NO_CHUNKS_IN_QUEUE             -15
+#define SCTP_INSTANCE_IN_USE                -16
 #define SCTP_SPECIFIC_FUNCTION_ERROR        1
 
 /* some constants for return codes from COMMUNICATION LOST or COMMUNICATION UP callbacks */
@@ -587,7 +587,7 @@ int sctp_setPathStatus(unsigned int associationID, short path_id, SCTP_PathStatu
 /*----------------------------------------------------------------------------------------------*/
 /*
  * These _could_ be build up from the above functions, but for the sake of a
- * complete API according to RFC 2960, section 10, we have the following
+ * complete API according to RFC 4960, section 10, we have the following
  * six functions  here, explicitly
  */
 
