@@ -1,47 +1,44 @@
-
 /*
- * $Id: streamengine.c 633 2005-05-19 22:17:21Z tuexen $
- * SCTP implementation according to RFC 2960.
+ * --------------------------------------------------------------------------
+ *
+ *           //=====   //===== ===//=== //===//  //       //   //===//
+ *          //        //         //    //    // //       //   //    //
+ *         //====//  //         //    //===//  //       //   //===<<
+ *              //  //         //    //       //       //   //    //
+ *       ======//  //=====    //    //       //=====  //   //===//
+ *
+ * -------------- An SCTP implementation according to RFC 4960 --------------
+ *
  * Copyright (C) 2000 by Siemens AG, Munich, Germany.
+ * Copyright (C) 2001-2004 Andreas Jungmaier
+ * Copyright (C) 2004-2026 Thomas Dreibholz
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
- *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Frderkennzeichen 01AK045).
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and the University of
+ * Duisburg-Essen, Institute for Experimental Mathematics, Computer
+ * Networking Technology group.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation, either version 2.1 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * There are two mailinglists available at http://www.sctp.de which should be
- * used for any discussion related to this implementation.
- *
- * Contact: discussion@sctp.de
+ * Contact: sctp-discussion@sctp.de
+ *          thomas.dreibholz@gmail.com
  *          tuexen@fh-muenster.de
- *          ajung@exp-math.uni-essen.de
- *
- *
- * Purpose: This modules implements the interface defined in streamengine.h and
- *          holds the private list of streams to process them for reasons of
- *          sending and receiving datachunks.
- *
+ *          andreas.jungmaier@web.de
  */
 
 #include <assert.h>
@@ -64,10 +61,10 @@
 
 typedef struct
 {
-    GList *pduList;          /* list of PDUs waiting for pickup (after notification has been called) */
-    GList *prePduList;       /* list of PDUs waiting for transfer to pduList and doing mdi arrive notification */
-    guint16 nextSSN;
-    guint16 highestSSN;      /* used to detect Protocol violations in se_searchReadyPdu */
+    GList   *pduList;         /* list of PDUs waiting for pickup (after notification has been called) */
+    GList   *prePduList;      /* list of PDUs waiting for transfer to pduList and doing mdi arrive notification */
+    guint16  nextSSN;
+    guint16  highestSSN;      /* used to detect Protocol violations in se_searchReadyPdu */
     gboolean highestSSNused;
     int index;
 }ReceiveStream;
@@ -79,8 +76,8 @@ typedef struct
 
 typedef struct
 {
-    unsigned int numSendStreams;
-    unsigned int numReceiveStreams;
+    unsigned int    numSendStreams;
+    unsigned int    numReceiveStreams;
     ReceiveStream*  RecvStreams;
     SendStream*     SendStreams;
     gboolean*       recvStreamActivated;
@@ -95,14 +92,14 @@ typedef struct
  */
 typedef struct _delivery_data
 {
-    guint8 chunk_flags;
+    guint8  chunk_flags;
     guint16 data_length;
     guint32 tsn;
     guint16 stream_id;
     guint16 stream_sn;
     guint32 protocolId;
-    guint32  fromAddressIndex;
-    guchar data[MAX_DATACHUNK_PDU_LENGTH];
+    guint32 fromAddressIndex;
+    guchar  data[MAX_DATACHUNK_PDU_LENGTH];
 }
 delivery_data;
 
@@ -219,6 +216,22 @@ void* se_new_stream_engine (unsigned int numberReceiveStreams,        /* max of
 }
 
 
+/* Free all chunks in list */
+static void free_delivery_pdu(gpointer list_element, gpointer user_data)
+{
+   delivery_pdu* d_pdu = (delivery_pdu*)list_element;
+   int           i;
+
+   if(d_pdu->ddata != NULL) {
+      for (i = 0; i < (int)d_pdu->number_of_chunks; i++) {
+         free(d_pdu->ddata[i]);
+         d_pdu->ddata[i] = NULL;
+      }
+      free(d_pdu->ddata);
+      free(d_pdu);
+   }
+}
+
 
 /* Deletes the instance pointed to by streamengine.
 */
@@ -232,14 +245,15 @@ se_delete_stream_engine (void *septr)
   event_log (INTERNAL_EVENT_0, "delete streamengine: freeing send streams");
   free(se->SendStreams);
 
-    for (i = 0; i < se->numReceiveStreams; i++) {
-        event_logi (VERBOSE, "delete streamengine: freeing data for receive stream %d",i);
-        /* whatever is still in these lists, delete it before freeing the lists */
-        g_list_foreach(se->RecvStreams[i].pduList, &free_list_element, NULL);
-        g_list_foreach(se->RecvStreams[i].prePduList, &free_list_element, NULL);
-        g_list_free(se->RecvStreams[i].pduList);
-        g_list_free(se->RecvStreams[i].prePduList);
-    }
+  for (i = 0; i < se->numReceiveStreams; i++) {
+     event_logi (VERBOSE, "delete streamengine: freeing data for receive stream %d",i);
+     /* whatever is still in these lists, delete it before freeing the lists */
+     g_list_foreach(se->RecvStreams[i].pduList, &free_delivery_pdu, NULL);
+     g_list_foreach(se->RecvStreams[i].prePduList, &free_delivery_pdu, NULL);
+     g_list_free(se->RecvStreams[i].pduList);
+     g_list_free(se->RecvStreams[i].prePduList);
+  }
+
   event_log (INTERNAL_EVENT_0, "delete streamengine: freeing receive streams");
   free(se->RecvStreams);
   free(se->recvStreamActivated);
@@ -313,7 +327,7 @@ se_ulpsend (unsigned short streamId, unsigned char *buffer,
     {
         error_logii (ERROR_MAJOR, "STREAM ID OVERFLOW in se_ulpsend: wanted %u, got only %u",
             streamId, se->numSendStreams);
-        mdi_sendFailureNotif (buffer, byteCount, context);
+        mdi_sendFailureNotif (buffer, byteCount, (unsigned int*)context);
         return SCTP_PARAMETER_PROBLEM;
     }
 
@@ -330,7 +344,7 @@ se_ulpsend (unsigned short streamId, unsigned char *buffer,
          if ((1 + fc_readNumberOfQueuedChunks()) > maxQueueLen) return SCTP_QUEUE_EXCEEDED;
        }
 
-        cdata = malloc(sizeof(chunk_data));
+        cdata = (chunk_data*)malloc(sizeof(chunk_data));
         if (cdata == NULL) {
             return SCTP_OUT_OF_RESOURCES;
         }
@@ -385,7 +399,7 @@ se_ulpsend (unsigned short streamId, unsigned char *buffer,
 
       for (i = 1; i <= numberOfSegments; i++)
       {
-            cdata = malloc(sizeof(chunk_data));
+            cdata = (chunk_data*)malloc(sizeof(chunk_data));
             if (cdata == NULL) {
                 /* FIXME: this is unclean, as we have already assigned some TSNs etc, and
                  * maybe queued parts of this message in the queue, this should be cleaned
@@ -504,7 +518,7 @@ short se_ulpreceivefrom(unsigned char *buffer, unsigned int *byteCount,
             oldQueueLen = se->queuedBytes;
             copiedBytes = 0;
 
-            d_pdu = g_list_nth_data (se->RecvStreams[streamId].pduList, 0);
+            d_pdu = (delivery_pdu*)g_list_nth_data (se->RecvStreams[streamId].pduList, 0);
 
             r_pos       = d_pdu->read_position;
             r_chunk     = d_pdu->read_chunk;
@@ -627,7 +641,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
 
     event_log (INTERNAL_EVENT_0, "SE_RECVDATACHUNK CALLED");
 
-    d_chunk = malloc (sizeof (delivery_data));
+    d_chunk = (delivery_data*)malloc (sizeof (delivery_data));
     if (d_chunk == NULL) return SCTP_OUT_OF_RESOURCES;
 
     datalength =  byteCount - FIXED_DATA_CHUNK_SIZE;
@@ -638,7 +652,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
         error_info.stream_id = htons(d_chunk->stream_id);
         error_info.reserved = htons(0);
 
-        scu_abort(ECC_INVALID_STREAM_ID, sizeof(error_info), (void*)&error_info);
+        scu_abort(ECC_INVALID_STREAM_ID, sizeof(error_info), (unsigned char*)&error_info);
         free(d_chunk);
         return SCTP_UNSPECIFIED_ERROR;
     }
@@ -646,7 +660,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
     d_chunk->tsn = ntohl (dataChunk->tsn);     /* for efficiency */
 
     if (datalength <= 0) {
-        scu_abort(ECC_NO_USER_DATA, sizeof(unsigned int), (void*)&(dataChunk->tsn));
+        scu_abort(ECC_NO_USER_DATA, sizeof(unsigned int), (unsigned char*)&(dataChunk->tsn));
 
         free(d_chunk);
         return SCTP_UNSPECIFIED_ERROR;
@@ -662,9 +676,10 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
 
 
     se->List = g_list_insert_sorted(se->List, d_chunk, (GCompareFunc) sort_tsn_se);
+    se->queuedBytes += datalength;
 
     se->recvStreamActivated[d_chunk->stream_id] = TRUE;
-    return 0;
+    return SCTP_SUCCESS;
 }
 
   int se_searchReadyPdu(StreamEngine* se)
@@ -683,7 +698,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
     guint32 itemPosition = 0;
     event_log (INTERNAL_EVENT_0, " ================> se_searchReadyPdu <=============== ");
     event_logi (VVERBOSE, "List has %u elements", g_list_length(se->List));
-    for (i = 0; i < se->numReceiveStreams; i++) {
+    for (i = 0; i < (int)se->numReceiveStreams; i++) {
         se->RecvStreams[i].highestSSN = 0;
         se->RecvStreams[i].highestSSNused = FALSE;
     }
@@ -779,7 +794,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
                 {
                       event_log (VVERBOSE, "handling complete PDU");
 
-                      d_pdu = malloc(sizeof(delivery_pdu));
+                      d_pdu = (delivery_pdu*)malloc(sizeof(delivery_pdu));
                       if (d_pdu == NULL) {
                           return SCTP_OUT_OF_RESOURCES;
                       }
@@ -790,7 +805,7 @@ int se_recvDataChunk (SCTP_data_chunk * dataChunk, unsigned int byteCount, unsig
                       d_pdu->chunk_position = 0;
                       d_pdu->total_length = 0;
 
-                      d_pdu->ddata = malloc(nrOfChunks*sizeof(delivery_data*));
+                      d_pdu->ddata = (delivery_data**)malloc(nrOfChunks*sizeof(delivery_data*));
                       if (d_pdu->ddata == NULL) {
                           free(d_pdu);
                           return SCTP_OUT_OF_RESOURCES;
@@ -975,7 +990,8 @@ int se_deliver_unreliably(unsigned int up_to_tsn, SCTP_forward_tsn_chunk* chk)
         {
              d_chunk = (delivery_data*)(tmp->data);
 
-             g_list_remove (se->List, d_chunk);
+             se->List = g_list_remove (se->List, d_chunk);
+             se->queuedBytes -= d_chunk->data_length;
              free(d_chunk);
              tmp = g_list_first(se->List);
         }
